package desktophe.dialog;

import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import desktophe.Messages;
import desktophe.Util;
import desktophe.model.IndexSettingsModel;

/**
 * CfbNXݒ_CAO
 * 
 * @author kohno
 */
public class IndexSettingDialog  extends Dialog {

    public static final int HORIZONTAL_SPACING = 3;
    public static final int MARGIN_WIDTH = 0;
    public static final int MARGIN_HEIGHT = 2;

    /** _CAO{ */
    private Shell shell_;
    
    /** C̃Rg[zu */
    private Composite body;

    /** OK{^ */
    private Button btnOk_;
    
    /** LZ{^ */
    private Button btnCancel_;
	
    /** tH_ꗗXgRg[ */
    private List listboxDirectory_;
	
	/** CfbNXf[^i[tH_ */
	private Text textIndexDataFolder_;

	/** ݒei[NX */
	private IndexSettingsModel indexSettings_ = null;
	private Button checkAddTextExtension_;
	
	/** ǉeLXgt@Cgq */
	private Text textAdditionalTextExtension_;
	private Button checkAddHtmlExtension_;

	/** ǉHTMLt@Cgq */
	private Text textAdditionalHtmlExtension_;
	private Button checkAddMimeExtension_;

	/** ǉMIMEt@Cgq */
	private Text textAdditionalMimeExtension_;
	
	/** [Firefox̃LbVCfbNX]̃`FbN{bNX */
	private Button checkIndexFirefoxCache_ = null;
	
	/** [Gۃ[̃f[^CfbNX]̃`FbN{bNX */
	private Button checkIndexHidemaruMail_ = null;
	
	
    /**
     * OK, LZ{^_CAO𐶐
     * @param parent _CAO̐eɂȂShell
     * @param title ^Cg
     * @param style VF̃X^CB[_ȃ_CAȌꍇ́A
     * SWT.DIALOG_TRIM | SWT.PRIMARY_MODALw肷B
     * @param okListener OK{^̃Xi
     */
    public IndexSettingDialog(Shell parent) {
    	super(parent, SWT.APPLICATION_MODAL);
    	
        shell_ = new Shell(parent, SWT.DIALOG_TRIM | SWT.RESIZE | SWT.APPLICATION_MODAL);
        createComponents(shell_);
        shell_.setSize(600, 450);
        Util.setShellCenter(shell_);
    }

	/**
     * _CAOɏ\f[^ݒ肷郁\bh
     * 
     * @param list _CAOɏ\f[^
     */
    public void setData(IndexSettingsModel setting) {
    	assert setting != null;
    	assert setting.getTargetFolderList() != null;
    	
    	textIndexDataFolder_.setText( setting.getIndexDataFolder() );
    	
    	for (int i=0; i<setting.getTargetFolderList().size(); i++) {
        	listboxDirectory_.add((String)setting.getTargetFolderList().get(i));
    	}
    	
    	checkAddTextExtension_.setSelection( setting.isAddTextExtension() );
    	textAdditionalTextExtension_.setEnabled( setting.isAddTextExtension() );
    	textAdditionalTextExtension_.setText( setting.getAdditionalTextExtension() );
    	checkAddHtmlExtension_.setSelection( setting.isAddHtmlExtension() );
    	textAdditionalHtmlExtension_.setEnabled( setting.isAddHtmlExtension() );
    	textAdditionalHtmlExtension_.setText( setting.getAdditionalHtmlExtension() );
    	checkAddMimeExtension_.setSelection( setting.isAddMimeExtension() );
    	textAdditionalMimeExtension_.setEnabled( setting.isAddMimeExtension() );
    	textAdditionalMimeExtension_.setText( setting.getAdditionalMimeExtension() );
    	
    	// Firefox̃LbV
    	checkIndexFirefoxCache_.setSelection( setting.isIndexFirefoxCache() );
    	
    	checkIndexHidemaruMail_.setSelection( setting.isIndexHidemaruMail() );
    }

    /**
     * _CAÕR|[lg𐶐郁\bh
     * 
     * @param shell _CAOShell
     */
	private void createComponents(Shell shell) {
        GridData gridDataFillBoth = new GridData(GridData.FILL_BOTH);			// AƂATCYgݒ
        GridData gridDataFillHorizontal = new GridData(GridData.FILL_HORIZONTAL);	// ɁATCYgݒ

        GridLayout layout = new GridLayout(1, false);
        shell.setLayout(layout);
        shell.setText(Messages.getString("IndexSettingDialog.Title")); //$NON-NLS-1$
        
        Composite top = createLeftAlignmentComposite();
        Label l = new Label(top, SWT.NULL);
        l.setText(Messages.getString("IndexSettingDialog.IndexDataStoreFolder")); //$NON-NLS-1$
        
        textIndexDataFolder_ = new Text(top, SWT.BORDER);
		textIndexDataFolder_.setLayoutData(gridDataFillBoth);
        
        Button btnChangeFolder = new Button(top, SWT.PUSH);
        btnChangeFolder.setText(Messages.getString("IndexSettingDialog.ChangeIndexDataStoreFolder")); //$NON-NLS-1$
        btnChangeFolder.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent arg0) {
				btnChangeFolder_widgetSelected(arg0);
			}

			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
        });

        Label l3 = new Label(shell, SWT.NULL);
        l3.setText(Messages.getString("IndexSettingDialog.IndexDataStoreFolderDesc")); //$NON-NLS-1$

        // E
        createHorizotalLine(shell);
        
        Label l2 = new Label(shell, SWT.NULL);
        l2.setText(Messages.getString("IndexSettingDialog.IndexingTargetFolder")); //$NON-NLS-1$

        // Body Composite
        body = new Composite(shell, SWT.NONE);
	    GridLayout glayout = new GridLayout(2, false);
	    body.setLayout(glayout);
	    
        body.setLayoutData(gridDataFillBoth);
        
        listboxDirectory_ = new List(body, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL);
        listboxDirectory_.setLayoutData(gridDataFillBoth);

        Composite dirbuttonArea = new Composite(body, SWT.NONE);
	    GridLayout dblayout = new GridLayout(1, false);
	    dirbuttonArea.setLayout(dblayout);
        
		Button buttonAddDirectory = new Button(dirbuttonArea, SWT.PUSH);
        buttonAddDirectory.setText(Messages.getString("IndexSettingDialog.IndexingTargetFolder.Add")); //$NON-NLS-1$
        buttonAddDirectory.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent arg0) {
				buttonAddDirectory_widgetSelected(arg0);
			}
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
        });
        
		Button buttonDeleteDirectory = new Button(dirbuttonArea, SWT.PUSH);
		buttonDeleteDirectory.setText(Messages.getString("IndexSettingDialog.IndexingTargetFolder.Remove")); //$NON-NLS-1$
		buttonDeleteDirectory.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent arg0) {
				buttonDeleteDirectory_widgetSelected(arg0);
			}
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
        });

        // E
        createHorizotalLine(shell);

        checkAddTextExtension_ = new Button(shell, SWT.CHECK);
		checkAddTextExtension_.setText(Messages.getString("IndexSettingDialog.Check.AddTextFileExtension")); //$NON-NLS-1$
		checkAddTextExtension_.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				checkAddTextExtension_widgetSelected(arg0);
			}
		});
        textAdditionalTextExtension_ = new Text(shell, SWT.BORDER);
		textAdditionalTextExtension_.setLayoutData(gridDataFillHorizontal);

        checkAddHtmlExtension_ = new Button(shell, SWT.CHECK);
		checkAddHtmlExtension_.setText(Messages.getString("IndexSettingDialog.Check.AddHtmlFileExtension")); //$NON-NLS-1$
		checkAddHtmlExtension_.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				checkAddHtmlExtension_widgetSelected(arg0);
			}
		});
        textAdditionalHtmlExtension_ = new Text(shell, SWT.BORDER);
		textAdditionalHtmlExtension_.setLayoutData(gridDataFillHorizontal);

        checkAddMimeExtension_ = new Button(shell, SWT.CHECK);
		checkAddMimeExtension_.setText(Messages.getString("IndexSettingDialog.Check.AddMimeFileExtension")); //$NON-NLS-1$
		checkAddMimeExtension_.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent arg0) {
				checkAddMimeExtension_widgetSelected(arg0);
			}
		});
        textAdditionalMimeExtension_ = new Text(shell, SWT.BORDER);
		textAdditionalMimeExtension_.setLayoutData(gridDataFillHorizontal);

        // E
        createHorizotalLine(shell);
		
		checkIndexFirefoxCache_ = new Button(shell, SWT.CHECK);
		checkIndexFirefoxCache_.setText(Messages.getString("IndexSettingDialog.Check.IndexFirefoxCache")); //$NON-NLS-1$
		
		checkIndexHidemaruMail_ = new Button(shell, SWT.CHECK);
		checkIndexHidemaruMail_.setText(Messages.getString("IndexSettingDialog.Check.IndexHidemarumail")); //$NON-NLS-1$

		// E
        createHorizotalLine(shell);

        // OK, LZ{^
        Composite bottom = createRightAlignmentComposite();
        btnOk_ = new Button(bottom, SWT.PUSH);
        btnOk_.setText(Messages.getString("IndexSettingDialog.OK")); //$NON-NLS-1$
        shell.setDefaultButton(btnOk_);
        btnCancel_ = new Button(bottom, SWT.PUSH);
        btnCancel_.setText(Messages.getString("IndexSettingDialog.Cancel")); //$NON-NLS-1$
        btnOk_.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent arg0) {
				btnOk_widgetSelected(arg0);
			}
			public void widgetDefaultSelected(SelectionEvent arg0) {
			}
        });
        btnCancel_.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent arg0) {
//                shell_.dispose();
                shell_.close();
            }
        });
    }

	/**
	 * 
	 * @param arg0
	 */
	protected void checkAddMimeExtension_widgetSelected(SelectionEvent arg0) {
		textAdditionalMimeExtension_.setEnabled( checkAddMimeExtension_.getSelection() );
	}

	/**
	 * 
	 * @param arg0
	 */
	protected void checkAddHtmlExtension_widgetSelected(SelectionEvent arg0) {
		textAdditionalHtmlExtension_.setEnabled( checkAddHtmlExtension_.getSelection() );
	}

	/**
	 * 
	 * @param arg0
	 */
	protected void checkAddTextExtension_widgetSelected(SelectionEvent arg0) {
		textAdditionalTextExtension_.setEnabled( checkAddTextExtension_.getSelection() );
	}

	/**
	 * CfbNXf[^i[tH_[ύX]{^ꂽƂɌĂ΂郁\bh
	 * 
	 * @param arg0 ZNVCxg
	 */
	protected void btnChangeFolder_widgetSelected(SelectionEvent arg0) {
    	DirectoryDialog dialog = new DirectoryDialog(shell_);
    	String path = dialog.open();
    	
    	if (path == null)  return;
    	
    	textIndexDataFolder_.setText(path);
	}

	/**
	 * [OK]{^ꂽƂɌĂ΂郁\bh
	 * 
	 * @param arg0 Cxg
	 */
    protected void btnOk_widgetSelected(SelectionEvent arg0) {
    	if (textIndexDataFolder_.getText().length() == 0) {
    	    MessageBox box1 = new MessageBox(shell_, SWT.OK | SWT.ICON_WARNING);
    		box1.setMessage(Messages.getString("IndexSettingDialog.Error.IndexDataStoreFolderIsNotSpecified")); //$NON-NLS-1$
    		box1.open();
    		
    		textIndexDataFolder_.setFocus();
    		return;
    	}
    	
    	if (listboxDirectory_.getItemCount() == 0) {
    	    MessageBox box1 = new MessageBox(shell_, SWT.OK | SWT.ICON_WARNING);
    		box1.setMessage(Messages.getString("IndexSettingDialog.Error.IndexingTargetFolderIsNotSpecified")); //$NON-NLS-1$
    		box1.open();
    		return;
    	}
    	
    	indexSettings_ = new IndexSettingsModel();
    	
    	indexSettings_.setIndexDataFolder_( textIndexDataFolder_.getText() );
    	
    	ArrayList directoryArray = new ArrayList();
    	for (int i=0; i<listboxDirectory_.getItems().length; i++) {
    		String dir = listboxDirectory_.getItems()[i];
    		directoryArray.add(dir);
    	}
    	
    	indexSettings_.setTargetFolderList_( directoryArray );
    	
    	indexSettings_.setAddTextExtension( checkAddTextExtension_.getSelection() );
    	indexSettings_.setAdditionalTextExtension( textAdditionalTextExtension_.getText() );
    	indexSettings_.setAddHtmlExtension( checkAddHtmlExtension_.getSelection() );
    	indexSettings_.setAdditionalHtmlExtension( textAdditionalHtmlExtension_.getText() );
    	indexSettings_.setAddMimeExtension( checkAddMimeExtension_.getSelection() );
    	indexSettings_.setAdditionalMimeExtension( textAdditionalMimeExtension_.getText() );
    	
    	// Firefox̃LbV
    	indexSettings_.setIndexFirefoxCache( checkIndexFirefoxCache_.getSelection() );
    	
    	indexSettings_.setIndexHidemaruMail( checkIndexHidemaruMail_.getSelection() );
    	
    	shell_.dispose();
	}

    /**
     * [폜]{^ꂽƂɌĂ΂郁\bh
     * 
     * @param arg0 Cxg
     */
	protected void buttonDeleteDirectory_widgetSelected(SelectionEvent arg0) {
		int index = listboxDirectory_.getSelectionIndex();
		
		if (index < 0)  return;
		
		listboxDirectory_.remove(index);
	}

	/**
	 * [ǉ]{^ꂽƂɌĂ΂郁\bh
	 * 
	 * @param arg0 Cxg
	 */
	protected void buttonAddDirectory_widgetSelected(SelectionEvent arg0) {
    	DirectoryDialog dialog = new DirectoryDialog(shell_);
    	String path = dialog.open();
    	
    	if (path == null)  return;
    	
    	listboxDirectory_.add(path);
	}

	/**
	 * w肳ꂽR|[lgɐ̋Eǉ郁\bh
	 * 
	 * @param c ǉΏۂ̃R|[lg
	 * @return ǉE
	 */
	private Label createHorizotalLine(Composite c) {
        Label line = new Label(c, SWT.SEPARATOR | SWT.HORIZONTAL);
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        line.setLayoutData(data);
        return line;
    }

	/**
	 * E񂹂̃R|[lg𐶐郁\bh
	 * 
	 * @return
	 */
    private Composite createRightAlignmentComposite() {
        Composite c = new Composite(shell_, SWT.NONE);
        GridLayout layout = new GridLayout(2, false);
        layout.horizontalSpacing = HORIZONTAL_SPACING;
        layout.marginWidth = MARGIN_WIDTH;
        layout.marginHeight = MARGIN_HEIGHT;
        c.setLayout(layout);
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_END);
        c.setLayoutData(data);
        return c;
    }

	/**
	 * E񂹂̃R|[lg𐶐郁\bh
	 * 
	 * @return
	 */
    private Composite createLeftAlignmentComposite() {
        Composite c = new Composite(shell_, SWT.NONE);
        GridLayout layout = new GridLayout(3, false);
        layout.horizontalSpacing = HORIZONTAL_SPACING;
        layout.marginWidth = MARGIN_WIDTH;
        layout.marginHeight = MARGIN_HEIGHT;
        c.setLayout(layout);
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
        c.setLayoutData(data);
        return c;
    }

    /**
     * _CAOJ\bh
     */
    public IndexSettingsModel open() {
        shell_.open();
        Display display = getParent().getDisplay();
        
    	while (!shell_.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
    	
    	return indexSettings_;
    }
}
