package desktophetest;

import java.util.Calendar;
import java.util.List;

import junit.framework.TestCase;
import desktophe.SearchStringParser;

/**
 * SearchStringParserNXɑ΂eXg
 * 
 * @author kohno
 */
public class SearchStringParserTest extends TestCase {

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(SearchStringParserTest.class);
	}

	/*
	 * 'desktophe.SearchStringParser.getPhrase()' ̂߂̃eXgE\bh
	 */
	public void testGetPhrase() {
		// Pꌟ
		{
			SearchStringParser parser = new SearchStringParser("hoge");
			assertEquals("hoge", parser.getPhrase());
		}

		// AND 
		{
			SearchStringParser parser = new SearchStringParser("hoge masu");
			assertEquals("hoge AND masu", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("aaa bbbb@cc");
			assertEquals("aaa AND bbbb AND cc", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("aaa  gg");
			assertEquals("aaa AND gg", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("s@{@kC");
			assertEquals("s AND { AND kC", parser.getPhrase());
		}

		// t[Y
		{
			SearchStringParser parser = new SearchStringParser("\"Hello, world\"");
			assertEquals("Hello, world", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("hoge \"Hello, world\"");
			assertEquals("hoge AND Hello, world", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("hoge \"Hello, world\" mogo");
			assertEquals("hoge AND Hello, world AND mogo", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("\"Hello, world");
			assertEquals("Hello, world", parser.getPhrase());
		}

		// NOT
		{
			SearchStringParser parser = new SearchStringParser("hoge -mao");
			assertEquals("hoge ANDNOT mao", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("hoge -mao zap");
			assertEquals("hoge ANDNOT mao AND zap", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("hoge \"-mao zap\"");
			assertEquals("hoge AND -mao zap", parser.getPhrase());
		}
	
		// OR
		{
			SearchStringParser parser = new SearchStringParser("hoge OR masu");
			assertEquals("hoge OR masu", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("t@OR@");
			assertEquals("t OR ", parser.getPhrase());
		}
		{
			SearchStringParser parser = new SearchStringParser("hoge OR masu lol");
			assertEquals("hoge OR masu AND lol", parser.getPhrase());
		}
		
		// Ov
		{
			SearchStringParser parser = new SearchStringParser("dog*");
			assertEquals("[BW] dog", parser.getPhrase());
		}
		
		// v
		{
			SearchStringParser parser = new SearchStringParser("*dog");
			assertEquals("[EW] dog", parser.getPhrase());
		}
	}

	/*
	 * 'desktophe.SearchStringParser.getAttributes()' ̂߂̃eXgE\bh
	 */
	public void testGetAttributes() {
		{
			SearchStringParser parser = new SearchStringParser("hoge name:.doc");
			assertEquals("hoge", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lfile ISTRINC .doc", ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("s path:obNAbv");
			assertEquals("s", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lreal ISTRINC obNAbv", ary[0]);
		}

		{
			SearchStringParser parser = new SearchStringParser("aaa bbb ccc");
			assertEquals("aaa AND bbb AND ccc", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNull(ary);
		}

		{
			SearchStringParser parser = new SearchStringParser("name:.xls");
			assertEquals("[UVSET]", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lfile ISTRINC .xls", ary[0]);
		}

		{
			SearchStringParser parser = new SearchStringParser("name:.xls path:dv");
			assertEquals("[UVSET]", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(2, ary.length);
			assertEquals("_lfile ISTRINC .xls", ary[0]);
			assertEquals("_lreal ISTRINC dv", ary[1]);
		}

		{
			SearchStringParser parser = new SearchStringParser("c from:yamada");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("from ISTRINC yamada", ary[0]);
		}

		{
			SearchStringParser parser = new SearchStringParser("c to:yamada");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("to ISTRINC yamada", ary[0]);
		}

		// t͈͎w̃eXg
		{
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			SearchStringParser parser = new SearchStringParser("c age:0day");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}

		{
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.add(Calendar.DAY_OF_MONTH, -2);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			SearchStringParser parser = new SearchStringParser("c age:2days");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}

		{
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.add(Calendar.MONTH, -1);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			SearchStringParser parser = new SearchStringParser("c age:0month");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}

		{
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.add(Calendar.MONTH, -11);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			SearchStringParser parser = new SearchStringParser("c age:10months");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}

		{
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.add(Calendar.YEAR, -1);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			SearchStringParser parser = new SearchStringParser("c age:0year");
			assertEquals("c", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}
		
		// datefrom
		{
			SearchStringParser parser = new SearchStringParser("bZ[W datefrom:2007/01/10");
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.set(Calendar.YEAR, 2007);
			cal.set(Calendar.MONTH, 0);
			cal.set(Calendar.DAY_OF_MONTH, 10);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			assertEquals("bZ[W", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("VN@datefrom:02/15");
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.set(Calendar.YEAR, 2007);
			cal.set(Calendar.MONTH, 1);
			cal.set(Calendar.DAY_OF_MONTH, 15);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			assertEquals("VN", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMGE " + timestr, ary[0]);
		}
		
		// dateto
		{
			SearchStringParser parser = new SearchStringParser("bZ[W dateto:2007/01/10");
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			cal.set(Calendar.YEAR, 2007);
			cal.set(Calendar.MONTH, 0);
			cal.set(Calendar.DAY_OF_MONTH, 10);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			assertEquals("bZ[W", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMLE " + timestr, ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("VN@dateto:02/15");
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			cal.set(Calendar.YEAR, 2007);
			cal.set(Calendar.MONTH, 1);
			cal.set(Calendar.DAY_OF_MONTH, 15);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			assertEquals("VN", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMLE " + timestr, ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("VN@dateto:1");
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			cal.set(Calendar.DAY_OF_MONTH, 1);
			long time = cal.getTimeInMillis() / 1000;
			String timestr = Long.toString(time);
			
			assertEquals("VN", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("@mdate NUMLE " + timestr, ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("VN@dateto:aaa");
			
			assertEquals("VN", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
//			assertEquals("@mdate NUMLE " + timestr, ary[0]);
			// GNZvVȂƂ̊mFBe͌؂ȂB
		}
		
		// under
		{
			SearchStringParser parser = new SearchStringParser("doc under:c:\\");
			
			assertEquals("doc", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lreal ISTRBW c:\\", ary[0]);
		}
		{
			SearchStringParser parser = new SearchStringParser("doc under:c:\\temp");
			
			assertEquals("doc", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lreal ISTRBW c:\\temp", ary[0]);
		}
		{
			SearchStringParser parser = 
				new SearchStringParser("doc spec under:\"C:\\Documents and Settings\"");
			
			assertEquals("doc AND spec", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lreal ISTRBW C:\\Documents and Settings", ary[0]);
		}
		{
			SearchStringParser parser = 
				new SearchStringParser("doc spec under:\"C:\\Documents and Settings\" \"I love you\"");
			
			assertEquals("doc AND spec AND I love you", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNotNull(ary);
			assertEquals(1, ary.length);
			assertEquals("_lreal ISTRBW C:\\Documents and Settings", ary[0]);
		}
	}

	/*
	 * 'desktophe.SearchStringParser.getHighlightWordList()' ̂߂̃eXgE\bh
	 */
	public void testGetHighlightWordList() {
		{
			SearchStringParser parser = new SearchStringParser("aaa");
			assertEquals("aaa", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNull(ary);
			
			List l = parser.getHighlightWordList();
			assertNotNull(l);
			assertEquals(1, l.size());
			
			String str = (String)l.get(0);
			assertEquals("aaa", str);
		}

		{
			SearchStringParser parser = new SearchStringParser("aaa bb");
			assertEquals("aaa AND bb", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNull(ary);
			
			List l = parser.getHighlightWordList();
			assertNotNull(l);
			assertEquals(2, l.size());
			
			String str = (String)l.get(0);
			assertEquals("aaa", str);

			str = (String)l.get(1);
			assertEquals("bb", str);
		}

		{
			SearchStringParser parser = new SearchStringParser("aaa bb@-cccc");
			assertEquals("aaa AND bb ANDNOT cccc", parser.getPhrase());
			
			String[] ary = parser.getAttributes();
			assertNull(ary);
			
			List l = parser.getHighlightWordList();
			assertNotNull(l);
			assertEquals(2, l.size());
			
			String str = (String)l.get(0);
			assertEquals("aaa", str);

			str = (String)l.get(1);
			assertEquals("bb", str);
		}
	}
}
