/*
 * Created on 2004/05/21
 *
 */
package hidemarumail;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;

import estraier.Condition;
import estraier.Database;
import estraier.Document;
import estraier.Result;

/**
 * f[^C|[^[NX
 */
public class DataImporter {
	/** vOCƎ̐ݒL^vpeBt@C̖O */
	private static final String PROPERTY_FILE_NAME = "hidemarumail.properties";
	
	/** \[Xt@C̃pXURIŋL^鑮 */
	static final String ATTR_SOURCE_FILE_URI = "source-file-uri";
	
	/** \[Xt@C̃pXlCeBut@CVXeŋL^鑮 */
	static final String ATTR_SOURCE_FILE_NATIVE = "_lreal";

	/** \[Xt@ĆAY[̃ItZbglL^鑮 */
	static final String ATTR_OFFSET = "offset";

	/** \[Xt@C̃pXlCeBut@CVXeŋL^鑮 */
	static public final String URI_PREFIX = "hidemarumail";

	/** Hyper Estraierf[^x[Xւ̎Q */
	private Database heDatabase_ = null;
	
	/** Gۃ[̃[gf[^tH_ */
	private String rootDir_ = null; 
	
	/** O̓ǂݎt@C̍ŏIXVL^ĂvpeB */
	private Properties propLastTimeParsedFiles_ = null;
	
	/** Aǂݎt@C̍ŏIXVL^vpeB */
	private Properties propThisTimeParsedFiles_ = null;
	

	/**
	 * f[^̃C|[gs\bh
	 * @param hedb Hyper Estraier̃f[^x[XBłɃI[vς݂Ȃ̂ŁAI[vKv͂ȂBN[YȂł悢B
	 * @param settingDataFolder ݒf[^ȂǂL^tH_Błɑ݂Ă邱Ƃۏ؂Ă
	 */
	public void importData(Database hedb, File settingDataFolder ) {
		assert settingDataFolder.isDirectory() : "ݒf[^ۑtH_݂Ȃ";
		
		heDatabase_ = hedb;

		// ܂Ałɓǂݍ񂾃t@CɂāAXVĂ邩A폜Ă΁ACfbNXf[^폜
		deleteModifiedData(settingDataFolder);
		
		rootDir_ = Util.getHidemaruMailDataFolder();
		File rootDirectory = new File(rootDir_);
		
		assert rootDirectory != null : "ǂݎ茳fBNgw肳ĂȂ" ;
		assert rootDirectory.isDirectory() : "w肳Ăǂݎ茳fBNg݂Ȃ";
		
		propThisTimeParsedFiles_ = new Properties();
		
		// fBNg\t@CtB^
		FileFilter directoryFilter = new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isDirectory();
			}
		};
		
		File[] directories = rootDirectory.listFiles(directoryFilter);
		
		for (int i=0; i<directories.length; i++) {
			importAccountData(directories[i]);
		}

		// ǂݎt@CƁAXVNL^
		try {
			FileOutputStream out = new FileOutputStream(settingDataFolder.getPath() + File.separator + PROPERTY_FILE_NAME);

			propThisTimeParsedFiles_.store(out, "");
			
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		System.out.println("HidemaruMail Index Plugin: Indexing finished.");
	}
	
	/**
	 * CfbNXAXVꂽt@CA폜ꂽt@C̃f[^폜郁\bh
	 *
	 */
	private void deleteModifiedData(File settingDataFolder) {
		String filepath = settingDataFolder.getPath() + File.separator + PROPERTY_FILE_NAME;
		propLastTimeParsedFiles_ = new Properties();

		// t@C݂Aǂݏo邱ƂmF
		if (!(new File(filepath).canRead())) {
			return;
		}

		FileInputStream in = null;
		
		try {
			in = new FileInputStream(filepath);
			propLastTimeParsedFiles_.load(in);
			Enumeration names = propLastTimeParsedFiles_.propertyNames();
			
			while (names.hasMoreElements()) {
				String name = (String) names.nextElement();
				File file = new File(name);
				
				if (!file.canRead()) {
					removeDataFromIndex(file.toURI());
				}
				
				String lastModifiedDate = new Date( file.lastModified() ).toString();
				String parsedLastModifiedDate = propLastTimeParsedFiles_.getProperty(name);
				
				assert parsedLastModifiedDate != null;
				
				if (!lastModifiedDate.equals( parsedLastModifiedDate ) ) {
					removeDataFromIndex(file.toURI());
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * w肳ꂽt@Cǂݎ[f[^ACfbNX폜郁\bh
	 * 
	 * @param sourceFileName
	 */
	private void removeDataFromIndex(URI fileUri) {
	    Condition cond = new Condition();
	    cond.set_phrase("[UVSET]");
	    
	    String attr = ATTR_SOURCE_FILE_URI + " STREQ " + fileUri.toString();
	    
	    cond.add_attr( attr );
	    Result result = heDatabase_.search(cond);
	    
	    System.out.println(fileUri.toString() + " is modified.");

	    for(int i = 0; i < result.doc_num(); i++){
	    	System.out.println("Removing ID: " + result.get_doc_id(i));
	    	heDatabase_.out_doc(result.get_doc_id(i), 0);
	    }
	}
	
	/**
	 * Gۃ[̊eAJEg̃f[^C|[g郁\bh
	 * 
	 * @param dir Gۃ[̃AJEg̃tH_pX
	 * @param db 
	 */
	private void importAccountData(File dir) {
		assert(dir != null);
		assert(dir.isDirectory());
		
		FileFilter directoryFilter = new FileFilter() {
			public boolean accept(File pathname) {
				if (!pathname.isDirectory())  return false;
				
				return pathname.getName().equals("User")
					|| pathname.getName().equals("M")
					|| pathname.getName().equals("Mς");
			}
		};

		File[] directories = dir.listFiles(directoryFilter);
		
		for (int i=0; i<directories.length; i++) {
			parseDirectory(directories[i]);
		}
	}
	
	/**
	 * w肳ꂽfBNg .txt t@C͂ATufBNgċNIɒׂ郁\bh
	 * @param directory fBNg
	 */
	private void parseDirectory(File directory) {
		assert directory.isDirectory();
		
		// .txt\t@CtB^
		FileFilter txtFilter = new FileFilter() {
			public boolean accept(File pathname) {
				if (pathname.isDirectory())  return false;
				
				return pathname.toString().endsWith(".txt");
			}
		};
		
		File[] txtFiles = directory.listFiles(txtFilter);
		
		for (int i=0; i<txtFiles.length; i++) {
			String path = txtFiles[i].getPath();
			String modifiedDate = new Date( txtFiles[i].lastModified() ).toString();
			String lastModifiedDate = propLastTimeParsedFiles_.getProperty(path);
			propThisTimeParsedFiles_.setProperty(txtFiles[i].getPath(), modifiedDate);
			
			if (lastModifiedDate == null || !lastModifiedDate.equals( modifiedDate )) {
				// t@C͂AHyper EstraierDocument`ɂ
				System.out.println("Parsing: " + path);
				List dataList = FileParser.parse(txtFiles[i], rootDir_);
				
				for (int j=0; j<dataList.size(); j++) {
			    	Document mailData = (Document)dataList.get(j);
					heDatabase_.put_doc(mailData, 0);
				}
			} else {
				// O͂ĂXVĂȂ
				System.out.println("Old file: " + path);
			}
		}

		// fBNg\t@CtB^
		FileFilter directoryFilter = new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isDirectory();
			}
		};
		
		File[] directories = directory.listFiles(directoryFilter);
		
		for (int i=0; i<directories.length; i++) {
			parseDirectory(directories[i]);
		}
	}
}

