package hidemarumail;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.regex.Pattern;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import estraier.Database;
import estraier.Document;

/**
 * 
 * @author kohno
 *
 */
public class Util {
	static {
	    System.loadLibrary("HidemaruMailPlugin");
	}

	public static native String getHidemaruMailDataFolder();

	public static native String getHidemaruMailProgramPath();
	
	/** t@CXgŕ\ACR */
	public static Image image_ = null;
	
	/**
	 * "@uri"ɐݒ肳ꂽƂɁAGۃ[̃f[^t@CY郁[̖{
	 * ǂݏoAԂ\bh
	 * 
	 * @param uri
	 * @return
	 */
	public static String getMailBodyText(Document doc) {
		assert doc != null;
		
		String uri = doc.attr("@uri");
		String filepath = doc.attr( DataImporter.ATTR_SOURCE_FILE_NATIVE );
		
		assert uri.startsWith("hidemarumail://") : "@urizOł: " + uri;
		assert filepath != null : DataImporter.ATTR_SOURCE_FILE_URI + "ݒ肳Ă܂B";
		
		String offsetstr = uri.substring(uri.lastIndexOf(':') + 1);
		int offset = Integer.parseInt(offsetstr);
		BufferedReader reader = null;
		StringBuffer bodyText = new StringBuffer();
		
		try {
			reader = new BufferedReader( new FileReader(filepath) );
			
			String line;
			int offsetsum = 0;
			boolean inbody = false;

			while ((line = reader.readLine()) != null) {
				if (offset <= offsetsum) {
					// ړĨ[ɒBƎvB{ǂݏo
					if (!inbody) {
						if (line.length() == 0) {
							inbody = true;
						}
						continue;
					} else {
						if (line.startsWith("!")) {
							// ̃[ɒB
							break;
						}
						
						bodyText.append(line);
						bodyText.append("\n");
					}
				} else {
					offsetsum += line.getBytes().length + 2;
				}
			}
		} catch (FileNotFoundException e) {
			return "G[Bt@Cǂݎ܂B\nłɍ폜Ă\܂B\n" + filepath;
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		return bodyText.toString();
	}
	
	/**
	 * 
	 * @param doc
	 * @param tree
	 * @param score
	 * @return
	 */
	public static TreeItem createTreeItemFromHeDocument(Document doc, Tree tree, int score) {
		TreeItem titem = new TreeItem(tree, SWT.NULL);
		String[] data = {
				doc.attr( Constants.ATTRIBUTE_TITLE ),
			Integer.toString( score ),
			convertSizeFormat( doc.attr("@size") ),
			doc.attr( Constants.ATTRIBUTE_DOCUMENT_TYPE ),
			convertW3CDTFDatetimeFormat(doc.attr( Constants.ATTRIBUTE_MODIFIED_DATE )),
			doc.attr( "folder" ),
		};
		titem.setText(data);
		titem.setData(Constants.DATA_DOCUMENT_ID, new Integer(doc.id()));
		titem.setData(Constants.ATTRIBUTE_URI, doc.attr( Constants.ATTRIBUTE_URI ));
		titem.setData(Constants.ATTRIBUTE_REAL_PATH, doc.attr( Constants.ATTRIBUTE_REAL_PATH ));
		
		if (image_ == null) {
			Program p = Program.findProgram(".eml");
			if (p != null) {
				ImageData imagedata = p.getImageData ();
				if (imagedata != null) {
					image_ = new Image(tree.getDisplay(), imagedata);
				}
			}
		}

		if (image_ != null)  titem.setImage(image_);

		return titem;
	}
	
	/**
	 * 
	 * @param db
	 * @param js
	 * @param parent
	 */
	public static void addShadowItemFromHeDocument(Database db, int[] js, TreeItem parent) {
		for (int i=0; i<js.length; i+=2) {
			Document doc = db.get_doc(js[i], 0);
			assert doc != null : "Document data is null";

			double score = js[i+1] >= 9999 ? 1.0 : js[i+1] / 10000.0;
			String scoreStr = Double.toString( score );
			
			if (scoreStr.indexOf('.') > 0 && scoreStr.length() > scoreStr.indexOf('.') + 4) {
				scoreStr = scoreStr.substring(0, scoreStr.indexOf('.') + 4);
			}

			TreeItem titem = new TreeItem(parent, SWT.NULL);
			String[] data = {
					doc.attr( Constants.ATTRIBUTE_TITLE ),
					scoreStr,
				convertSizeFormat( doc.attr("@size") ),
				doc.attr( Constants.ATTRIBUTE_DOCUMENT_TYPE ),
				convertW3CDTFDatetimeFormat(doc.attr( Constants.ATTRIBUTE_MODIFIED_DATE )),
				doc.attr( "folder" ),
			};
			titem.setText(data);
			titem.setData(Constants.DATA_DOCUMENT_ID, new Integer(doc.id()));
			titem.setData(Constants.ATTRIBUTE_URI, doc.attr( Constants.ATTRIBUTE_URI ));
			titem.setData(Constants.ATTRIBUTE_REAL_PATH, doc.attr( Constants.ATTRIBUTE_REAL_PATH ));
			
			if (image_ != null)  titem.setImage(image_);
		}
	}

	/**
	 * Hyper Estraier̃TCYAGNXv[̏ɕϊ郁\bh
	 * 
	 * @param str ϊ
	 * @return ϊ㕶
	 */
	static public String convertSizeFormat(String str) {
		if (str == null)  return "";
		if (str.length() <= 3)  return "1 KB";
		
		String num = str.substring(0, str.length() - 3);
		NumberFormat formatter = NumberFormat.getNumberInstance();
		return formatter.format(Long.parseLong(num)) + " KB";
	}

	/**
	 * ݒt@Ci[tH_̃pX𕶎ŕԂ\bh.
	 * Xy[X܂ޏꍇ́A[ " łĕԂB
	 * 
	 * @return
	 */
	public static String getSettingDataFolder() {
		String filepath = System.getProperty("user.home");
		filepath += File.separator + "Application Data" + File.separator + "DesktopHE";

		if (filepath.indexOf(' ') >= 0) {
			filepath = "\"" + filepath + "\"";
		}
		
		return filepath;
	}

	/**
	 * Hyper Estraier̓t`AGNXv[̏ɕϊ郁\bh
	 * 
	 * @param str ϊ
	 * @return ϊ㕶
	 */
	static public String convertW3CDTFDatetimeFormat(String str) {
		if (str == null || str.length() == 0)  return "";
		if (str.indexOf('T') < 0)  return Util.convertRFC822DatetimeFormat(str);
		
		String date = str.substring(0, str.indexOf('T'));
		String[] dateAry = date.split("-");
		
		if (dateAry == null || dateAry.length < 3)  return Util.convertRFC822DatetimeFormat(str);
		
		String time = str.substring(str.indexOf('T') + 1);
		String[] timeAry = time.split(":");
		
		if (timeAry == null || timeAry.length < 2)  return Util.convertRFC822DatetimeFormat(str);
		
		return dateAry[0] + "/" + dateAry[1] + "/" + dateAry[2] + " " + timeAry[0] + ":" + timeAry[1];
	}

	/**
	 * 󂯎ACalendarɕϊĕԂ\bh
	 * @param str
	 * @return
	 */
	private static String convertRFC822DatetimeFormat(String str) {
		int cpos = str.indexOf(',');
		if (cpos != -1) {
			str = str.substring(cpos + 1).trim();
		}
		
		Pattern pattern = Pattern.compile(" ");
		String[] strs = pattern.split(str.trim());
		
		String y, m, d, h, s;
		
		// uNvA2̏ꍇ悤Ȃ̂ŁA2Ȃ瓪"20"t
		if (strs[2].length() == 2)  strs[2] = "20" + strs[2];
		
		y = strs[2];
		m = convertMonth(strs[1]);
		d = strs[0];
		
		pattern = Pattern.compile(":");
		String[] strs2 = pattern.split(strs[3]);
		
		h = strs2[0];
		s = strs2[1];
		
		return y + "/" + m + "/" + d + " " + h + ":" + s;
	}

	/**
	 * 󂯎ɑΉuv̐lԂ\bh
	 * @param s uv̕
	 * @return uv̐l
	 */
	private static String convertMonth(String s) {
		if (s.equals("Jan")) {
			return "01";
		}
		else if (s.equals("Feb")) {
			return "02";
		}
		else if (s.equals("Mar")) {
			return "03";
		}
		else if (s.equals("Apr")) {
			return "04";
		}
		else if (s.equals("May")) {
			return "05";
		}
		else if (s.equals("Jun")) {
			return "06";
		}
		else if (s.equals("Jul")) {
			return "07";
		}
		else if (s.equals("Aug")) {
			return "08";
		}
		else if (s.equals("Sep")) {
			return "09";
		}
		else if (s.equals("Oct")) {
			return "10";
		}
		else if (s.equals("Nov")) {
			return "11";
		}
		else if (s.equals("Dec")) {
			return "12";
		}
	
		throw new StringIndexOutOfBoundsException();
	}
}
