#include "desktophe_Util.h"
#include <windows.h>
#include <shlobj.h>
#include <assert.h>
#include <string>

#define IDM_OPEN_FOLDER 1000

static const char g_szClassName[] = "DesktopHE_ContextMenu"; 			///< EBhENX

static HINSTANCE 		g_hInstance = NULL;
static HWND				g_hWnd = NULL;
static LPCONTEXTMENU2 	g_pContMenu2 = NULL; 

//
// Entry Point
//
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    switch(fdwReason) {
	case DLL_PROCESS_ATTACH:
		g_hInstance = hinstDLL;
		return TRUE;
	
	case DLL_PROCESS_DETACH:
		break;
    }
    return FALSE;
}

HRESULT DoHandleMenuMsg(UINT wMessage, WPARAM wParam, LPARAM lParam)
{
    if (g_pContMenu2){
        return g_pContMenu2->HandleMenuMsg(wMessage, wParam, lParam);
    }
	else {
        return -1;
    }
}

/**
 * EBhEvV[W
 * @param hWnd [in] Handle to the window.
 * @param msg [in] Specifies the message.
 * @param wp [in] Specifies additional message information. The contents of this parameter depend on the value of the uMsg parameter. 
 * @param lp [in] Specifies additional message information. The contents of this parameter depend on the value of the uMsg parameter.
 * @return The return value is the result of the message processing and depends on the message sent.
 */
LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp) 
{ 
	switch (msg) { 
	case WM_COMMAND: 
		switch (LOWORD(wp)) { 
			case IDM_OPEN_FOLDER:
				break;
		} 
		break; 
		
	case WM_CLOSE: 
		DestroyWindow(hWnd); 
		break; 
	
	case WM_DESTROY: 
		PostQuitMessage(0); 
		break; 

	case WM_INITMENUPOPUP:
//		OutputDebugString("WM_INITMENUPOPUP");
		DoHandleMenuMsg(msg, wp, lp);
		return  FALSE;
	
	case WM_DRAWITEM:
	case WM_MEASUREITEM:
//		OutputDebugString("WM_DRAWITEM or WM_MEASUREITEM");
		DoHandleMenuMsg(msg, wp, lp);
		return  TRUE;
		
	default: 
		return (DefWindowProc(hWnd, msg, wp, lp)); 
	} 
	
	return 0; 
}


/**
 * EBhEENX̓o^s֐
 * @param hInst CX^X̃nh
 * @return o^ꂽNXӓIɎʂAg
 */
ATOM InitApp(HINSTANCE hInst) 
{ 
	WNDCLASSEX wc; 
	ZeroMemory(&wc, sizeof(wc));
	wc.cbSize = sizeof(WNDCLASSEX); 
	wc.lpfnWndProc = WndProc; 				//vV[W 
	wc.cbClsExtra = 0; 
	wc.cbWndExtra = 0; 
	wc.hInstance = hInst;					//CX^X 
	wc.hCursor = LoadCursor(NULL, IDC_ARROW); 
	wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH); 
	wc.lpszClassName = (LPCSTR)g_szClassName; 
	
	//ACR 
	return (RegisterClassEx(&wc)); 
}

/**
 * EBhE̐s֐
 * @param hInst AvP[ṼCX^X̃nh
 * @param nCmdShow 
 * @return EBhE̐ɐTRUEAȊȌꍇFALSE
 */
BOOL InitInstance(HINSTANCE hInst) 
{ 
	g_hWnd = CreateWindow(g_szClassName, 
		"DesktopHE ContextMenu", 				//^Cgo[ɂ̖O\܂ 
		WS_OVERLAPPEDWINDOW, 	//EBhE̎ 
		CW_USEDEFAULT, 			//wW 
		CW_USEDEFAULT, 			//xW 
		300, 		// 
		250, 		// 
		NULL, 		//eEBhẼnhAeƂNULL 
		NULL, 		//j[nhANXj[gƂNULL 
		hInst, 		//CX^Xnh 
		NULL); 
		
	if (!g_hWnd) return FALSE; 
	
	ShowWindow(g_hWnd, SW_HIDE); 
	UpdateWindow(g_hWnd); 
	return TRUE; 
}



JNIEXPORT jboolean JNICALL Java_desktophe_Util_isIndexing(JNIEnv *, jclass)
{
	HANDLE han = NULL; //쒆~[ebNX 
	
	if ((han = CreateMutex(NULL, false, "DesktopHE_Indexer_Mutex")) == NULL 
		|| GetLastError() == ERROR_ALREADY_EXISTS) { //dÑ`FbNA쒆 
		
		if(han)  CloseHandle(han); //dN̎̏ 
		return JNI_TRUE;
	}	
	
	if(han) { 
		ReleaseMutex(han);
		CloseHandle(han);
	}
	
	return JNI_FALSE;
}

/**
 * GNXv[Ɠl̃ReLXgj[\郁\bh
 */
JNIEXPORT void JNICALL Java_desktophe_Util_showContextMenu(JNIEnv* env, jclass obj, jstring jsMessage)
{
	assert(g_hInstance != NULL);
	
	// ܂Aj[R}hΉp̃EBhE
	InitApp(g_hInstance);
	InitInstance(g_hInstance); 
	
	jsize nUniLength = env->GetStringLength(jsMessage);
	const jchar* pszUnicode = env->GetStringChars(jsMessage, NULL);
	
	HRESULT hResult;
	IShellFolder* pDesktopFolder = NULL;

	// ܂fXNgbṽVF擾
	hResult = SHGetDesktopFolder(&pDesktopFolder);
	if (hResult != NOERROR) {
		OutputDebugString("SHGetDesktopFolder() Error");
		MessageBox(NULL, "SHGetDesktopFolder() Error", "error", MB_OK);
		
		DestroyWindow(g_hWnd);
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}

	LPITEMIDLIST pItemIdList = NULL;
	
	// t@CpX񂩂AITEMIDLIST擾
	hResult = pDesktopFolder->ParseDisplayName(NULL, NULL, (wchar_t*)pszUnicode, NULL, &pItemIdList, NULL);
	if (hResult != NOERROR) {
		OutputDebugString("pDesktopFolder->ParseDisplayName() Error");

		pDesktopFolder->Release();
		DestroyWindow(g_hWnd);
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}
	
	std::string targetFileName;
	std::string targetFolder;
	
	// ϊĂ邱ƂmFASCIIŃtH_pXƃt@Ci[
	{
		TCHAR nativePath[512];
		SHGetPathFromIDList(pItemIdList, nativePath);
		OutputDebugString(nativePath);
		
		std::string targetFilePath = nativePath;
		targetFileName = targetFilePath.substr(targetFilePath.rfind("\\") + 1, targetFilePath.length() - targetFilePath.rfind("\\"));
		OutputDebugString(targetFileName.c_str());
		
		targetFolder = targetFilePath.substr(0, targetFilePath.rfind("\\"));
		OutputDebugString(targetFolder.c_str());
	}
	
    LPITEMIDLIST    pIDLFolder;
	
	// tH_̃ACeIDXg擾
	{
		std::wstring filepathString((wchar_t*)pszUnicode); 
		int pos = filepathString.rfind(L"\\");
		filepathString = filepathString.substr(0, pos);

		wchar_t folder[512];
		wcscpy(folder, filepathString.c_str());
	    hResult = pDesktopFolder->ParseDisplayName( NULL, NULL, folder, NULL, &pIDLFolder, NULL);
		if (hResult != S_OK) {
			OutputDebugString("pDesktopFolder->ParseDisplayName() Error");
			pDesktopFolder->Release();
			env->ReleaseStringChars(jsMessage, pszUnicode);
			return;	
		}
	}
	
	// ϊĂ邱ƂmF
//	{
//		TCHAR nativePath[512];
//		SHGetPathFromIDList(pIDLFolder, nativePath);
//		OutputDebugString(nativePath);
//	}

	// tH_̃ACeIDXgAShellFolderC^[tF[X擾
	LPSHELLFOLDER pFolder;
	hResult = pDesktopFolder->BindToObject(pIDLFolder, NULL, IID_IShellFolder, (void**)&pFolder);
	if (hResult != S_OK) {
		OutputDebugString("pDesktopFolder->BindToObject() Error");
		pDesktopFolder->Release();
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}

	// t@C񋓂	
	IEnumIDList* pEnumIDList = NULL;
	hResult = pFolder->EnumObjects(NULL, SHCONTF_NONFOLDERS, &pEnumIDList);
	if (hResult != S_OK) {
		OutputDebugString("pFolder->EnumObjects() Error");
		pDesktopFolder->Release();
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}
	
	LPITEMIDLIST pTargetItemIdList = NULL;
	
	if (pEnumIDList != NULL) {
		// 
		LPMALLOC pMalloc;
	
		// IMallocC^[tFCXւ̃|C^擾܂B
		hResult = SHGetMalloc( &pMalloc );

		while (true) {	
			LPITEMIDLIST pTempItemIdList = NULL;
			hResult = pEnumIDList->Next(1, &pTempItemIdList, NULL);
			
			if (hResult != S_OK)  break;
			
			TCHAR nativePath[512];
			SHGetPathFromIDList(pTempItemIdList, nativePath);
			OutputDebugString(nativePath);
			
			std::string tempFilePath = nativePath;
			std::string tempFileName = tempFilePath.substr(tempFilePath.rfind("\\") + 1, tempFilePath.length() - tempFilePath.rfind("\\"));
			OutputDebugString(tempFileName.c_str());

			if (tempFileName == targetFileName) {
				pTargetItemIdList = pTempItemIdList;
				break;
			}

			pMalloc->Free( pTempItemIdList );
		}
		
		pMalloc->Release();	
	}
	
	if (pTargetItemIdList == NULL)  return;
	
	LPCONTEXTMENU pContMenu;
	HMENU hMenu;
	POINT cMenuPt;
	
	hResult = pFolder->GetUIObjectOf(NULL, 1, (LPCITEMIDLIST *)&pTargetItemIdList, IID_IContextMenu, 
										0, (LPVOID *)&pContMenu); 
	if (hResult != S_OK) {
		OutputDebugString("pDesktopFolder->GetUIObjectOf() Error");
		pDesktopFolder->Release();
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}
	
	hResult = pContMenu->QueryInterface(IID_IContextMenu2, (LPVOID*)&g_pContMenu2); 
	if (hResult != S_OK) {
		OutputDebugString("pContMenu->QueryInterface() Error");
		pDesktopFolder->Release();
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}
	
	hMenu = CreatePopupMenu();

	hResult = g_pContMenu2->QueryContextMenu(hMenu, 0, 1, 0x7fff, CMF_NORMAL);
	if (!(SUCCEEDED(hResult))) {
		OutputDebugString("pContMenu2->QueryContextMenu() Error");
		pDesktopFolder->Release();
		env->ReleaseStringChars(jsMessage, pszUnicode);
		return;	
	}
	
	// ReLXgj[ɁA"1̃tH_J"ǉ
	{
		MENUITEMINFO mii;
		memset(&mii, 0, sizeof(mii));	
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_FTYPE;
		mii.fType = MFT_SEPARATOR;
	
		InsertMenuItem(hMenu, 0, TRUE, &mii);	

		memset(&mii, 0, sizeof(mii));	
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_ID | MIIM_TYPE;
		mii.fType = MFT_STRING;
		mii.wID = IDM_OPEN_FOLDER;
		mii.dwTypeData = "1̃tH_J";
	
		InsertMenuItem(hMenu, 0, TRUE, &mii);	
	}

	POINT point;
	GetCursorPos(&point);
	
	cMenuPt.x = point.x;
	cMenuPt.y = point.y;

	DWORD iCommand = TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_LEFTBUTTON, 
					cMenuPt.x, cMenuPt.y, 0, g_hWnd, NULL); 

//	{
//		char buf[512];
//		wsprintf(buf, "TrackPopupMenu(): %d", iCommand);
//		OutputDebugString(buf);
//	}
	
	if (iCommand > 0) {
		if (iCommand == IDM_OPEN_FOLDER) {
			ShellExecute(NULL, "open", targetFolder.c_str(), NULL, NULL, SW_SHOWNORMAL);
		}
		else {
			CMINVOKECOMMANDINFO stInvokeInfo;
			stInvokeInfo.cbSize = sizeof(CMINVOKECOMMANDINFO);
			stInvokeInfo.fMask = 0;
			stInvokeInfo.hwnd = g_hWnd;
			stInvokeInfo.lpVerb = MAKEINTRESOURCE(iCommand-1);
			stInvokeInfo.lpParameters = NULL;
			stInvokeInfo.lpDirectory = NULL;
			stInvokeInfo.nShow = SW_SHOWNORMAL;
			stInvokeInfo.dwHotKey = 0;
			stInvokeInfo.hIcon = NULL;
			pContMenu->InvokeCommand(&stInvokeInfo);
		}
	} 
	
	// ȍ~AI	
	
	DestroyMenu(hMenu);
	DestroyWindow(g_hWnd);
	
	// 
	LPMALLOC pMalloc;

	// IMallocC^[tFCXւ̃|C^擾܂B
	hResult = SHGetMalloc( &pMalloc );
	if (hResult == NOERROR ) {
		// [X܂B
		pMalloc->Free( pItemIdList );
		pMalloc->Free( pTargetItemIdList );
		
		pMalloc->Release();	
	}
	
	pDesktopFolder->Release();
	
	env->ReleaseStringChars(jsMessage, pszUnicode);
}

/**
 * WXgJRẼfBNgǂݏo֐
 */
static std::string GetJRENDirectory()
{
	HKEY hkey;
	LONG retCode;
	
	retCode = RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\JavaSoft\\Java Runtime Environment", 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_READ, NULL, &hkey, NULL);
		
	if (retCode != ERROR_SUCCESS) {
		return "";
	}
	
	BYTE data[1024] = {0};
	DWORD size = 1024;
	retCode = RegQueryValueEx(hkey, "CurrentVersion", NULL, NULL, data, &size);
	
	if (retCode != ERROR_SUCCESS) {
		return "";
	}
	
    // TuL[񋓂
	TCHAR bufKeyName[256];
	DWORD nameLen;
	FILETIME time;
	LONG ret;
	float versionNum = 0.0f;
	std::string versionStr = "";
	
	for (int index=0; ; index++) {
		nameLen = 256;
    	ret = RegEnumKeyEx(hkey, index, bufKeyName, &nameLen, NULL, NULL, NULL, &time); 	
		
		if (ret == ERROR_NO_MORE_ITEMS)  break;
		
		if (atof(bufKeyName) > versionNum) {
		    versionNum = atof(bufKeyName);	
			versionStr = bufKeyName;
		}
	}
	
    RegCloseKey(hkey);

	if (versionStr.length() == 0) {
		return "";	
	}
	
    std::string regKey = "SOFTWARE\\JavaSoft\\Java Runtime Environment\\";
	regKey += versionStr;

	retCode = RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey.c_str(), 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_READ, NULL, &hkey, NULL);
		
	if (retCode != ERROR_SUCCESS) {
		return "";
	}
	
	size = 1024;
	RegQueryValueEx(hkey, "JavaHome", NULL, NULL, data, &size);
    RegCloseKey(hkey);
	
	return (char*)data;
}

/**
 * CXg[ĂJREWXgǂݍ݁Ajava.exẽtpXԂ
 */
JNIEXPORT jstring JNICALL Java_desktophe_Util_getJavaExePath(JNIEnv* env, jclass obj)
{
	std::string javaExePath = GetJRENDirectory() + "\\bin\\java";
	
	jstring result = env->NewStringUTF( javaExePath.c_str() );
	return result;
}
